@@ -6,24 +6,22 @@ use Apache::Table ();
 
 {
     no strict;
-    $VERSION = '0.31';
+    $VERSION = '0.33';
     @ISA = qw(Apache);
     __PACKAGE__->mod_perl::boot($VERSION);
 }
 
 #just prototype methods here, moving to xs later
-sub param {
-    my $self = shift;
-    my($name, $value) = @_;
-    my $tab = $self->parms;
-    unless ($name) {
-	my %seen;
-	return wantarray ? grep { !$seen{$_}++ } keys %$tab : $tab;
+
+sub instance {
+    my $class = shift;
+    my $r = shift;
+    if (my $apreq = $r->pnotes('apreq')) {
+        return $apreq;
     }
-    if (defined $value) {
-	$tab->set($name, $value);
-    }
-    return wantarray ? ($tab->get($name)) : scalar $tab->get($name);
+    my $new_req = $class->new($r, @_);
+    $r->pnotes('apreq', $new_req);
+    return $new_req;
 }
 
 1;
@@ -43,13 +41,13 @@ Apache::Request - Methods for dealing with client request data
 I<Apache::Request> is a subclass of the I<Apache> class, which adds methods
 for parsing B<GET> requests and B<POST> requests where I<Content-type>
 is one of I<application/x-www-form-urlencoded> or 
-I<multipart/form-data>.
+I<multipart/form-data>. See the libapreq(3) manpage for more details.
 
 =head1 Apache::Request METHODS
 
 =over 4
 
-=item new
+=head2 new
 
 Create a new I<Apache::Request> object with an I<Apache> request_rec object:
 
@@ -89,9 +87,68 @@ error code if a file upload is attempted:
      return $status;
  }
 
+=item TEMP_DIR
+
+Sets the directory where upload files are spooled.  On a *nix-like
+that supports link(2), the TEMP_DIR should be placed on the same
+file system as the final destination file:
+
+ my $apr = Apache::Request->new($r, TEMP_DIR => "/home/httpd/tmp");
+ my $upload = $apr->upload('file');
+ $upload->link("/home/user/myfile") || warn "link failed: $!";
+
+=item HOOK_DATA
+
+Extra configuration info to be passed to and upload hook.
+See next item for details.
+
+=item UPLOAD_HOOK
+
+Sets up a callback to run whenever file upload data is read. This
+can be used to provide an upload progress meter during file uploads.
+Apache will automatically continue writing the original data to
+$upload->fh after the hook exits.
+
+ my $transparent_hook = sub {
+   my ($upload, $buf, $len, $hook_data) = @_;
+   warn "$hook_data: got $len bytes for " . $upload->name;
+ };
+
+ my $apr = Apache::Request->new($r, 
+                                HOOK_DATA => "Note",
+                                UPLOAD_HOOK => $transparent_hook,
+                               );
+ $apr->parse;
+
 =back
 
-=item parse
+=head2 instance
+
+The instance() class method allows Apache::Request to be a singleton.
+This means that whenever you call Apache::Request->instance() within a
+single request you always get the same Apache::Request object back.
+This solves the problem with creating the Apache::Request object twice
+within the same request - the symptoms being that the second
+Apache::Request object will not contain the form parameters because
+they have already been read and parsed.
+
+  my $apr = Apache::Request->instance($r, DISABLE_UPLOADS => 1);
+
+Note that C<instance()> call will take the same parameters as the above
+call to C<new()>, however the parameters will only have an effect the
+first time C<instance()> is called within a single request. Extra
+parameters will be ignored on subsequent calls to C<instance()> within
+the same request.
+
+Subrequests receive a new Apache::Request object when they call
+instance() - the parent request's Apache::Request object is not copied
+into the subrequest.
+
+Also note that it is unwise to use the C<parse()> method when using
+C<instance()> because you may end up trying to call it twice, and
+detecting errors where there are none.
+
+=head2 parse
 
 The I<parse> method does the actual work of parsing the request.
 It is called for you by the accessor methods, so it is not required but
@@ -107,7 +164,7 @@ occur:
 	return $status; 
     } 
 
-=item param
+=head2 param
 
 Get or set request parameters:
 
@@ -116,7 +173,7 @@ Get or set request parameters:
     my @params = $apr->param;
     $apr->param('foo' => [qw(one two three)]);
 
-=item upload
+=head2 upload
 
 Returns a single I<Apache::Upload> object in a scalar context or
 all I<Apache::Upload> objects in an array context: 
@@ -140,19 +197,19 @@ object associated with the given name:
 
 =over 4
 
-=item name
+=head2 name
 
 The name of the filefield parameter:
 
     my $name = $upload->name;
 
-=item filename
+=head2 filename
 
 The filename of the uploaded file:
 
     my $filename = $upload->filename;
 
-=item fh
+=head2 fh
 
 The filehandle pointing to the uploaded file:
 
@@ -161,13 +218,13 @@ The filehandle pointing to the uploaded file:
 	...
     }
 
-=item size
+=head2 size
 
 The size of the file in bytes:
 
     my $size = $upload->size;
 
-=item info
+=head2 info
 
 The additional header information for the uploaded file.
 Returns a hash reference tied to the I<Apache::Table> class.
@@ -181,7 +238,7 @@ a given header rather than a hash reference.  Examples:
 
     my $val = $upload->info("Content-type");
 
-=item type
+=head2 type
 
 Returns the I<Content-Type> for the given I<Apache::Upload> object:
 
@@ -189,7 +246,7 @@ Returns the I<Content-Type> for the given I<Apache::Upload> object:
     #same as
     my $type = $upload->info("Content-Type");
 
-=item next
+=head2 next
 
 As an alternative to using the I<Apache::Request> I<upload> method in
 an array context:
@@ -204,8 +261,28 @@ an array context:
 	...
     }
 
+=head2 tempname
+
+Provides the name of the spool file.
+
+=head2 link
+
+To avoid recopying the spool file on a Unix-like system,
+I<link> will create a hard link to it:
+
+  my $upload = $apr->upload('file');
+  $upload->link("/path/to/newfile") or
+      die sprintf "link from '%s' failed: $!", $upload->tempname;
+
+Typically the new name must lie on the same file system as the
+spool file. Check your system's link(2) manpage for details.
+
 =back
 
+=head1 SEE ALSO
+
+libapreq(3)
+
 =head1 CREDITS
 
 This interface is based on the original pure Perl version by Lincoln Stein.
